﻿using System;
using System.Collections.Generic;
using System.IO;

namespace graph
{
    public class Graph
    {
       public static int[,] afterReconstruction = null;

        private static int FindMinimumSpanningTree(List<Edge> edges, int[] tree, List<Edge> mpd, int treesCount)
        {
            int len = edges.Count;
            for (int i= 0; i < len; i++)
            {
                if (tree[edges[i].FirstNode] == 0)
                {
                    if (tree[edges[i].LastNode] == 0)
                    {
                        tree[edges[i].FirstNode] = tree[edges[i].LastNode] = treesCount;
                        treesCount++;
                    }
                    else
                    {
                        tree[edges[i].FirstNode] = tree[edges[i].LastNode];
                    }
                    mpd.Add(edges[i]);
                }
                else
                {
                    if (tree[edges[i].LastNode] == 0)
                    {
                        tree[edges[i].LastNode] = tree[edges[i].FirstNode];
                        mpd.Add(edges[i]);
                    }
                    else if (tree[edges[i].LastNode] != tree[edges[i].FirstNode]) // combine the trees
                    {
                        int oldTreeNumber = tree[edges[i].LastNode];

                        for (int j = 0; j < tree.Length; j++)
                        {
                            if (tree[j] == oldTreeNumber)
                            {
                                tree[j] = tree[edges[i].FirstNode];
                            }
                        }
                        mpd.Add(edges[i]);
                    }
                }
            }
            return treesCount;
        }

        private static void SaveTheTree(List<Edge> usedEdges)
        {
            var writer = new StreamWriter("graph.out");
            using (writer)
            {
                int len = usedEdges.Count;
                for (int i = 0; i < len; i++)
                {
                    writer.WriteLine(usedEdges[i].Number);
                }
            }
        }

        static int Main(string[] args)
        {
            var reader = new StreamReader("graph.in");
            int towns = 0;
            int roads = 0;
            List<Edge> edges = new List<Edge>();

            
            using (reader)
            {
                string line = reader.ReadLine();
                int counter = 0;

                while (line != null)
                {
                    counter++;
                    string[] curLine = line.Split(' ');
                    if (counter == 1)
                    {
                        towns = int.Parse(curLine[0]);
                        roads = int.Parse(curLine[1]);
                        afterReconstruction = new int[towns, towns];
                    }
                    else if (counter <= roads + 1)
                    {
                        int first = int.Parse(curLine[0]);
                        int second = int.Parse(curLine[1]);
                        int dist = int.Parse(curLine[2]);
                        edges.Add(new Edge(first, second, dist, counter - 1));
                    }
                    else
                    {
                        for (int i = 0; i < towns; i++)
                        {
                            afterReconstruction[counter - roads - 2, i] = int.Parse(curLine[i]);
                        }
                    }
                    line = reader.ReadLine();
                }
            }



            edges.Sort();

            int[] tree = new int[roads + 1];

            List<Edge> mpd = new List<Edge>();
            int treesCount = 1;
            treesCount = FindMinimumSpanningTree(edges, tree, mpd, treesCount);
            SaveTheTree(mpd);

            return 0;
        }
    }

    class Edge : IComparable<Edge>
    {
        private int firstNode;
        private int lastNode;
        private int distance;
        private int number;

        public int FirstNode
        {
            get
            {
                return this.firstNode;
            }
            set
            {
                this.firstNode = value;
            }
        }

        public int LastNode
        {
            get
            {
                return this.lastNode;
            }
            set
            {
                this.lastNode = value;
            }
        }

        public int Distance
        {
            get
            {
                return this.distance;
            }
            set
            {
                this.distance = value;
            }
        }

        public int Number
        {
            get
            {
                return this.number;
            }
            set
            {
                this.number = value;
            }
        }

        public Edge(int firstNode, int lastNode, int distance, int number)
        {
            this.FirstNode = firstNode;
            this.LastNode = lastNode;
            this.Distance = distance;
            this.Number = number;
        }

        public int CompareTo(Edge other)
        {
            int weightCompared = this.Distance.CompareTo(other.Distance);

            if (weightCompared == 0)
            {
                weightCompared = this.lastNode.CompareTo(other.lastNode);

                if (weightCompared == 0)
                {
                    return this.firstNode.CompareTo(other.firstNode);
                }
              //  return Graph.afterReconstruction[other.firstNode - 1, other.lastNode - 1].CompareTo(Graph.afterReconstruction[this.firstNode - 1, this.lastNode - 1]);
            }
            return weightCompared;
        }
    }

}